@@ -1,13 +1,13 @@
-import collections
+import collections.abc
 import types
 import unittest
-from test.support import get_c_recursion_limit
+from test.support import skip_emscripten_stack_overflow, skip_wasi_stack_overflow, exceeds_recursion_limit
 
 class TestExceptionGroupTypeHierarchy(unittest.TestCase):
     def test_exception_group_types(self):
-        self.assertTrue(issubclass(ExceptionGroup, Exception))
-        self.assertTrue(issubclass(ExceptionGroup, BaseExceptionGroup))
-        self.assertTrue(issubclass(BaseExceptionGroup, BaseException))
+        self.assertIsSubclass(ExceptionGroup, Exception)
+        self.assertIsSubclass(ExceptionGroup, BaseExceptionGroup)
+        self.assertIsSubclass(BaseExceptionGroup, BaseException)
 
     def test_exception_is_not_generic_type(self):
         with self.assertRaisesRegex(TypeError, 'Exception'):
@@ -193,77 +193,6 @@ class StrAndReprTests(unittest.TestCase):
                       "MyEG('flat', [ValueError(1), TypeError(2)]), "
                       "TypeError(2)])"))
 
-    def test_exceptions_mutation(self):
-        class MyEG(ExceptionGroup):
-            pass
-
-        excs = [ValueError(1), TypeError(2)]
-        eg = MyEG('test', excs)
-
-        self.assertEqual(repr(eg), "MyEG('test', [ValueError(1), TypeError(2)])")
-        excs.clear()
-
-        # Ensure that clearing the exceptions sequence doesn't change the repr.
-        self.assertEqual(repr(eg), "MyEG('test', [ValueError(1), TypeError(2)])")
-
-        # Ensure that the args are still as passed.
-        self.assertEqual(eg.args, ('test', []))
-
-        excs = (ValueError(1), KeyboardInterrupt(2))
-        eg = BaseExceptionGroup('test', excs)
-
-        # Ensure that immutable sequences still work fine.
-        self.assertEqual(
-            repr(eg),
-            "BaseExceptionGroup('test', (ValueError(1), KeyboardInterrupt(2)))"
-        )
-
-        # Test non-standard custom sequences.
-        excs = collections.deque([ValueError(1), TypeError(2)])
-        eg = ExceptionGroup('test', excs)
-
-        self.assertEqual(
-            repr(eg),
-            "ExceptionGroup('test', deque([ValueError(1), TypeError(2)]))"
-        )
-        excs.clear()
-
-        # Ensure that clearing the exceptions sequence doesn't change the repr.
-        self.assertEqual(
-            repr(eg),
-            "ExceptionGroup('test', deque([ValueError(1), TypeError(2)]))"
-        )
-
-    def test_repr_raises(self):
-        class MySeq(collections.abc.Sequence):
-            def __init__(self, raises):
-                self.raises = raises
-
-            def __len__(self):
-                return 1
-
-            def __getitem__(self, index):
-                if index == 0:
-                    return ValueError(1)
-                raise IndexError
-
-            def __repr__(self):
-                if self.raises:
-                    raise self.raises
-                return None
-
-        seq = MySeq(None)
-        with self.assertRaisesRegex(
-            TypeError,
-            r"__repr__ returned non-string \(type NoneType\)"
-        ):
-            ExceptionGroup("test", seq)
-
-        seq = MySeq(ValueError)
-        with self.assertRaises(ValueError):
-            BaseExceptionGroup("test", seq)
-
-
 
 def create_simple_eg():
     excs = []
@@ -531,15 +460,19 @@ class ExceptionGroupSplitTests(ExceptionGroupTestBase):
 class DeepRecursionInSplitAndSubgroup(unittest.TestCase):
     def make_deep_eg(self):
         e = TypeError(1)
-        for i in range(get_c_recursion_limit() + 1):
+        for i in range(exceeds_recursion_limit()):
             e = ExceptionGroup('eg', [e])
         return e
 
+    @skip_emscripten_stack_overflow()
+    @skip_wasi_stack_overflow()
     def test_deep_split(self):
         e = self.make_deep_eg()
         with self.assertRaises(RecursionError):
             e.split(TypeError)
 
+    @skip_emscripten_stack_overflow()
+    @skip_wasi_stack_overflow()
     def test_deep_subgroup(self):
         e = self.make_deep_eg()
         with self.assertRaises(RecursionError):
@@ -881,8 +814,8 @@ class NestedExceptionGroupSplitTest(ExceptionGroupSplitTestBase):
         eg = ExceptionGroup("eg", [ValueError(1), TypeError(2)])
         eg.__notes__ = 123
         match, rest = eg.split(TypeError)
-        self.assertFalse(hasattr(match, '__notes__'))
-        self.assertFalse(hasattr(rest, '__notes__'))
+        self.assertNotHasAttr(match, '__notes__')
+        self.assertNotHasAttr(rest, '__notes__')
 
     def test_drive_invalid_return_value(self):
         class MyEg(ExceptionGroup):
